@@ -228,7 +228,7 @@ st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet
 		stop("`promote_to_multi' should have length one, and applies to all geometry columns")
 
 	x = CPL_read_ogr(dsn, layer, query, as.character(options), quiet, type, fid_column_name,
-		drivers, wkt_filter, promote_to_multi, int64_as_string, dsn_exists, dsn_isdb)
+		drivers, wkt_filter, promote_to_multi, int64_as_string, dsn_exists, dsn_isdb, getOption("width"))
 	process_cpl_read_ogr(x, quiet, check_ring_dir = check_ring_dir,
 		stringsAsFactors = stringsAsFactors, geometry_column = geometry_column, ...)
 }
@@ -251,7 +251,7 @@ read_sf <- function(..., quiet = TRUE, stringsAsFactors = FALSE, as_tibble = TRU
 }
 
 clean_columns = function(obj, factorsAsCharacter) {
-	permitted = c("character", "integer", "numeric", "Date", "POSIXct", "logical")
+	permitted = c("character", "integer", "numeric", "Date", "POSIXct", "logical", "list")
 	for (i in seq_along(obj)) {
 		if (is.factor(obj[[i]])) {
 			obj[[i]] = if (factorsAsCharacter)
@@ -279,6 +279,11 @@ clean_columns = function(obj, factorsAsCharacter) {
 		# nocov end
 	}
 	colclasses = vapply(obj, function(x) permitted[ which(inherits(x, permitted, which = TRUE) > 0)[1] ] , "")
+	# check that list columns contain raw vectors:
+	for (lc in which(colclasses == "list")) {
+		if (!all(sapply(obj[[lc]], inherits, "raw")))
+			stop("list columns are only allowed with raw vector contents")
+	}
 	structure(obj, colclasses = colclasses)
 }
 
@@ -309,31 +314,41 @@ abbreviate_shapefile_names = function(x) {
 #'
 #' Write simple features object to file or database
 #' @param obj object of class \code{sf} or \code{sfc}
-#' @param dsn data source name (interpretation varies by driver - for some drivers, dsn is a file name, but may also be a
-#' folder or contain a database name) or a Database Connection (currently
-#' official support is for RPostgreSQL connections)
-#' @param layer layer name (varies by driver, may be a file name without extension); if layer is missing, the
-#' \link{basename} of \code{dsn} is taken.
+#' @param dsn data source name. Interpretation varies by driver: can be
+#'   a filename, a folder, a database name, or a Database Connection
+#'   (we officially test support for
+#'   \code{\link[RPostgres:Postgres]{RPostgres::Postgres()}} connections).
+#' @param layer layer name. Varies by driver, may be a file name without
+#'   extension; for database connection, it is the name of the table. If layer
+#'   is missing, the \code{basename} of \code{dsn} is taken.
 #' @param driver character; name of driver to be used; if missing and \code{dsn} is not a Database Connection, a driver name is guessed from \code{dsn};
 #' \code{st_drivers()} returns the drivers that are available with their properties; links to full driver documentation
-#' are found at \url{https://gdal.org/ogr_formats.html}.
+#' are found at \url{https://gdal.org/drivers/vector/index.html}
 #' @param ... other arguments passed to \link{dbWriteTable} when \code{dsn} is a
 #' Database Connection
 #' @param dataset_options character; driver dependent dataset creation options;
 #' multiple options supported.
 #' @param layer_options character; driver dependent layer creation options;
 #' multiple options supported.
-#' @param quiet logical; suppress info on name, driver, size and spatial reference
-#' @param factorsAsCharacter logical; convert \code{factor} objects into
-#' character strings (default), else into numbers by \code{as.numeric}.
+#' @param quiet logical; suppress info on name, driver, size and spatial
+#'   reference
+#' @param factorsAsCharacter logical; convert \code{factor} levels to character
+#'   strings (\code{TRUE}, default), otherwise into numbers when
+#'   factorsAsCharacter is \code{FALSE}. For database connections,
+#'   \code{factorsAsCharacter} is always \code{TRUE}.
 #' @param append logical; should we append to an existing layer, or replace it?
-#' if \code{TRUE} append, if \code{FALSE} replace; default \code{NA} will
-#' raise an error if the layer exists. See also next two arguments.
+#' if \code{TRUE} append, if \code{FALSE} replace.
+#' The default for \code{st_write} is \code{NA} which raises an error if the layer exists.
+#' The default for \code{write_sf} is \code{FALSE}, which overwrites any existing data.
+#' See also next two arguments for more control on overwrite behavior.
 #' @param delete_dsn logical; delete data source \code{dsn} before attempting
 #' to write?
 #' @param delete_layer logical; delete layer \code{layer} before attempting to
 #' write?
+#' The default for \code{st_write} is \code{FALSE} which raises an error if the layer exists.
+#' The default for \code{write_sf} is \code{TRUE}.
 #' @param fid_column_name character, name of column with feature IDs; if
+#' @param config_options character, named vector with GDAL config options
 #' specified, this column is no longer written as feature attribute.
 #' @details
 #' Columns (variables) of a class not supported are dropped with a warning.
@@ -346,9 +361,8 @@ abbreviate_shapefile_names = function(x) {
 #' When deleting layers or data sources is not successful, no error is emitted.
 #' \code{delete_dsn} and \code{delete_layer} should be
 #' handled with care; the former may erase complete directories or databases.
-#' @seealso \link{st_drivers}
-#' @return \code{obj}, invisibly; in case \code{obj} is of class \code{sfc},
-#' it is returned as an \code{sf} object.
+#' @seealso \link{st_drivers}, \link{dbWriteTable}
+#' @return \code{obj}, invisibly
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
 #' st_write(nc, paste0(tempdir(), "/", "nc.shp"))
@@ -377,6 +391,7 @@ st_write.sfc = function(obj, dsn, layer, ...) {
 		st_write.sf(st_sf(geom = obj), dsn, ...)
 	else
 		st_write.sf(st_sf(geom = obj), dsn, layer, ...)
+	invisible(obj)
 }
 
 #' @name st_write
@@ -385,8 +400,9 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 		driver = guess_driver_can_write(dsn),
 		dataset_options = NULL, layer_options = NULL, quiet = FALSE, factorsAsCharacter = TRUE,
 		append = NA, delete_dsn = FALSE, delete_layer = !is.na(append) && !append,
-		fid_column_name = NULL) {
+		fid_column_name = NULL, config_options = character(0)) {
 
+	return_obj = ret = obj
 	if (!is.null(list(...)$update)) {
 		.Deprecated("append", old = "update") # deprecated at 0.9-0
 		if (is.na(append))
@@ -412,10 +428,17 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 			append = FALSE
 		}
 
+		# check arguments
+		cl <- as.list(match.call())[-1L]
+		if ("overwrite" %in% names(cl)) {
+			stop("Argument `overwrite` in `st_write()` is deprecated, use `delete_layer` instead.", call. = FALSE)
+		}
+
+		check_append_delete(append, delete_layer)
 		dbWriteTable(dsn, name = layer, value = obj,
 					 append = append, overwrite = delete_layer,
 					 factorsAsCharacter = factorsAsCharacter, ...)
-		return(invisible(obj))
+		return(invisible(return_obj))
 	} else if (!inherits(dsn, "character")) { # add methods for other dsn classes here...
 		stop(paste("no st_write method available for dsn of class", class(dsn)[1]))
 	}
@@ -436,6 +459,10 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 	if (write_geometries <- inherits(obj, "sf")) {
 		geom = st_geometry(obj)
 		obj[[attr(obj, "sf_column")]] = NULL
+		if (driver == "GPKG" && is.na(st_crs(geom))) {
+			message('writing GPKG: substituting LOCAL_CS["Undefined Cartesian SRS"] for missing CRS')
+			st_crs(geom) = st_crs("LOCAL_CS[\"Undefined Cartesian SRS\"]")
+		}
 	} else { # create fake geometries:
 		v = vector("list", nrow(obj))
 		v[seq_len(nrow(obj))] = list(st_point())
@@ -466,39 +493,40 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 
 	ret = CPL_write_ogr(obj, dsn, layer, driver,
 		as.character(dataset_options), as.character(layer_options),
-		geom, dim, fids, quiet, append, delete_dsn, delete_layer,
-		write_geometries)
+		geom, dim, fids, config_options, quiet, append, delete_dsn, delete_layer,
+		write_geometries, getOption("width"))
 	if (ret == 1) { # try through temp file:
 		tmp = tempfile(fileext = paste0(".", tools::file_ext(dsn))) # nocov start
 		if (!quiet)
 			message(paste("writing first to temporary file", tmp))
 		if (CPL_write_ogr(obj, tmp, layer, driver,
 				as.character(dataset_options), as.character(layer_options),
-				geom, dim, fids, quiet, append, delete_dsn, delete_layer,
-				write_geometries) == 1)
+				geom, dim, fids, config_options, quiet, append, delete_dsn, delete_layer,
+				write_geometries, getOption("width")) == 1)
 			stop(paste("failed writing to temporary file", tmp))
 		if (!file.copy(tmp, dsn, overwrite = append || delete_dsn || delete_layer))
 			stop(paste("copying", tmp, "to", dsn, "failed"))
 		if (!file.remove(tmp))
 			warning(paste("removing", tmp, "failed"))
 	} # nocov end
-	invisible(obj)
+	invisible(return_obj)
 }
 
 #' @name st_write
 #' @export
 st_write.data.frame <- function(obj, dsn, layer = NULL, ...) {
-    sf = try(st_as_sf(obj), silent = TRUE)
+	sf = try(st_as_sf(obj), silent = TRUE)
 	if (!inherits(sf, "try-error"))
-    	st_write.sf(sf, dsn = dsn, layer = layer, ...)
+		st_write.sf(sf, dsn = dsn, layer = layer, ...)
 	else
-    	st_write.sf(obj, dsn = dsn, layer = layer, ...)
+		st_write.sf(obj, dsn = dsn, layer = layer, ...)
+	invisible(obj)
 }
 
 #' @name st_write
 #' @export
-write_sf <- function(..., quiet = TRUE, append = FALSE, delete_layer = TRUE) {
-	st_write(..., quiet = quiet, append = append, delete_layer = TRUE)
+write_sf <- function(..., quiet = TRUE, append = FALSE, delete_layer = !append) {
+	st_write(..., quiet = quiet, append = append, delete_layer = delete_layer)
 }
 
 #' Get GDAL drivers
@@ -518,7 +546,7 @@ write_sf <- function(..., quiet = TRUE, append = FALSE, delete_layer = TRUE) {
 #' @examples
 #' st_drivers()
 st_drivers = function(what = "vector") {
-	ret = CPL_get_rgdal_drivers(0)
+	ret = CPL_get_gdal_drivers(0)
 	row.names(ret) = ret$name
 	switch(what,
 		vector = ret[ret$is_vector,],
@@ -683,3 +711,18 @@ prefix_map <- list(
 #' Drivers for which update should be \code{TRUE} by default
 #' @docType data
 db_drivers <- c(unlist(prefix_map), "GPKG", "SQLite")
+
+# Utils ----------------------------------------------------------------------
+check_append_delete <- function(append, delete) {
+	if (append && delete) {
+		stop("`delete_layer` and `append` cannot both be `TRUE`", call. = FALSE)
+	}
+}
+
+#' @name st_write
+#' @export
+#' @details st_delete deletes layer(s) in a data source, or a data source if layers are 
+#' omitted; it returns TRUE on success, FALSE on failure, invisibly.
+st_delete = function(dsn, layer = character(0), driver = guess_driver_can_write(dsn), quiet = FALSE) {
+	invisible(CPL_delete_ogr(dsn, layer, driver, quiet) == 0)
+}
