@@ -40,15 +40,9 @@
 #'    Polygon(y[5:1,]), Polygon(y1), Polygon(x1), Polygon(y3)), "ID1")
 #' p2 = Polygons(list( Polygon(z[5:1,]), Polygon(z2), Polygon(z3), Polygon(z1)),
 #'   "ID2")
-#' if (require("rgeos")) {
-#'   r = createSPComment(SpatialPolygons(list(p1,p2)))
-#'   comment(r)
-#'   comment(r@polygons[[1]])
-#'   scan(text = comment(r@polygons[[1]]), quiet = TRUE)
-#'   library(sf)
-#'   a = st_as_sf(r)
-#'   summary(a)
-#' }
+#' r = SpatialPolygons(list(p1,p2))
+#' a = suppressWarnings(st_as_sf(r))
+#' summary(a)
 #' demo(meuse, ask = FALSE, echo = FALSE)
 #' summary(st_as_sf(meuse))
 #' summary(st_as_sf(meuse.grid))
@@ -145,9 +139,25 @@ st_as_sfc.SpatialLines = function(x, ..., precision = 0.0, forceMulti = FALSE) {
 st_as_sfc.SpatialPolygons = function(x, ..., precision = 0.0, forceMulti = FALSE) {
 	lst = if (forceMulti || any(sapply(x@polygons, function(x) moreThanOneOuterRing(x@Polygons)))) {
 		if (is.null(comment(x)) || comment(x) == "FALSE") {
-			if (!requireNamespace("rgeos", quietly = TRUE))
-				stop("package rgeos required for finding out which hole belongs to which exterior ring")
-			x = rgeos::createSPComment(x)
+#			if (!requireNamespace("rgeos", quietly = TRUE))
+#				stop("package rgeos required for finding out which hole belongs to which exterior ring")
+#			x = rgeos::createSPComment(x)
+			# https://github.com/r-spatial/sf/pull/1869/files/7f1921c9acc1000b92a81b3a0aa7126330d4ef12..cfa303c8fcdd0b9a7ea33eae402c1135bb8e50ba :
+			# warning("no comment found showing which hole belongs to which exterior ring")
+			# (warning causes revdep problem in pkg amt)
+			process_pl_comment <- function(pl) {
+				ID <- slot(pl, "ID")
+				crds <- lapply(slot(pl, "Polygons"), function(xx) slot(xx, "coords"))
+				raw <- st_sfc(st_polygon(crds))
+				val <- st_make_valid(raw)
+				if (inherits(val, "sfc_GEOMETRYCOLLECTION"))
+					val = st_collection_extract(val, "POLYGON")
+				res <- slot(as(val, "Spatial"), "polygons")[[1]]
+				slot(res, "ID") <- ID
+				res
+			}
+			slot(x, "polygons") <- lapply(slot(x, "polygons"), process_pl_comment)
+			comment(x) <- "TRUE"
 		}
 		lapply(x@polygons, function(y)
 			st_multipolygon(Polygons2MULTIPOLYGON(y@Polygons, comment(y))))
@@ -338,14 +348,12 @@ setAs("crs", "CRS", function(from) CRS_from_crs(from))
 CRS_from_crs = function(from) {
 	if (! requireNamespace("sp", quietly = TRUE))
 		stop("package sp required, please install it first")
-	ret = if (is.na(from))
-			sp::CRS(NA_character_)
-		else
-			sp::CRS(from$proj4string)
-        if (CPL_proj_version() >= "6.0.0" && 
-                    CPL_gdal_version() >= "3.0.0") {
-	    if (!is.null(from$wkt) && !is.na(from$wkt))
-		comment(ret) = comment(sp::CRS(SRS_string=from$wkt))
-        }
-	ret
+	nm <- "CRS"
+	attr(nm, "package") <- "sp" # See ?new:
+	obj <- new(nm, projargs = from$proj4string)
+	if (!is.na(from$wkt) && CPL_proj_version() >= "6.0.0" && CPL_gdal_version() >= "3.0.0")
+		comment(obj) <- from$wkt
+	obj
+	# we don't use sp::CRS(SRS_string = from$wkt) as rgdal may not be available,
+	# which would break, and from$wkt has already been validated by GDAL:
 }
