@@ -188,6 +188,15 @@ rename.sf <- function(.data, ...) {
 		stop("internal error: can't find `agr` columns") # nocov
 
 	vars_loc = loc[loc %in% agr_loc]
+
+	# https://github.com/r-spatial/sf/issues/1472
+	# but only fixes for the single sfc column case
+	sfcs = which(sapply(.data, inherits, "sfc"))
+	if (length(vars_loc) == 1 && any(vars_loc > sfcs[1])) {
+		w = which(vars_loc > sfcs)
+		vars_loc[w] = vars_loc[w] - 1
+	}
+
 	names(agr)[vars_loc] = names(vars_loc)
 
 	sf_column_loc_loc = match(sf_column_loc, loc)
@@ -234,6 +243,8 @@ summarise.sf <- function(.data, ..., .dots, do_union = TRUE, is_coverage = FALSE
 	ret = NextMethod()
 	if (!missing(do_union))
 		ret$do_union = NULL
+	if (!missing(is_coverage))
+		ret$is_coverage = NULL
 
 	if (! any(sapply(ret, inherits, what = "sfc"))) {
 		geom = if (inherits(.data, "grouped_df") || inherits(.data, "grouped_dt")) {
@@ -281,9 +292,13 @@ distinct.sf <- function(.data, ..., .keep_all = FALSE) {
 	if (!requireNamespace("rlang", quietly = TRUE))
 		stop("rlang required: install first?")
 
-	.data = dplyr::distinct(.data, ..., !! rlang::sym(sf_column), .keep_all = .keep_all)
-	.data[[ sf_column ]] = geom[ .data[[ sf_column ]] ]
-	st_as_sf(.data)
+	.data = dplyr::distinct(.data, ..., .keep_all = .keep_all)
+	if (is.null(.data[[ sf_column ]]))
+		.data
+	else {
+		.data[[ sf_column ]] = geom[ .data[[ sf_column ]] ]
+		st_as_sf(.data, sf_column_name = sf_column)
+	}
 }
 
 ## tidyr methods:
@@ -314,6 +329,40 @@ gather.sf <- function(data, key, value, ..., na.rm = FALSE, convert = FALSE, fac
 		sf_column_name = attr(data, "sf_column"))
 }
 
+pivot_longer.sf <- function (data, cols, names_to = "name", names_prefix = NULL,
+		names_sep = NULL, names_pattern = NULL, names_ptypes = NULL,
+		names_transform = NULL, names_repair = "check_unique",
+		values_to = "value", values_drop_na = FALSE, values_ptypes = NULL,
+		values_transform = NULL, ...) {
+
+  sf_column_name = attr(data, "sf_column")
+  data = as.data.frame(data)
+
+# instead of:
+#	st_as_sf(NextMethod(), sf_column_name = sf_column_name)
+# we avoid NextMethod(); for the reason
+# see https://github.com/tidyverse/tidyr/issues/1171:
+  if (!requireNamespace("tidyr", quietly = TRUE))
+    stop("tidyr required: install first?")
+  out <- tidyr::pivot_longer(
+    data = data,
+    cols = {{ cols }},
+    names_to = names_to,
+    names_prefix = names_prefix,
+    names_sep = names_sep,
+    names_pattern = names_pattern,
+    names_ptypes = names_ptypes,
+    names_transform = names_transform,
+    names_repair = names_repair,
+    values_to = values_to,
+    values_drop_na = values_drop_na,
+    values_ptypes = values_ptypes,
+    values_transform = values_transform,
+    ...
+  )
+  st_as_sf(out, sf_column_name = sf_column_name)
+}
+
 
 #' @name tidyverse
 #' @param fill see original function docs
@@ -333,7 +382,7 @@ spread.sf <- function(data, key, value, fill = NA, convert = FALSE, drop = TRUE,
 	value = rlang::enquo(value)
 
 	class(data) <- setdiff(class(data), "sf")
-    st_as_sf(tidyr::spread(data, !!key, !!value, fill = fill, convert = convert,
+	st_as_sf(tidyr::spread(data, !!key, !!value, fill = fill, convert = convert,
 		drop = drop, sep = sep), sf_column_name = attr(data, "sf_column"))
 }
 
@@ -499,6 +548,7 @@ register_all_s3_methods = function() {
 	register_s3_method("dplyr", "transmute", "sf")
 	register_s3_method("dplyr", "ungroup", "sf")
 	register_s3_method("tidyr", "gather", "sf")
+	register_s3_method("tidyr", "pivot_longer", "sf")
 	register_s3_method("tidyr", "spread", "sf")
 	register_s3_method("tidyr", "nest", "sf")
 	register_s3_method("tidyr", "separate", "sf")
@@ -508,22 +558,22 @@ register_all_s3_methods = function() {
 	register_s3_method("pillar", "obj_sum", "sfc")
 	register_s3_method("pillar", "type_sum", "sfc")
 	register_s3_method("pillar", "pillar_shaft", "sfc")
-	register_s3_method("spatstat", "as.ppp", "sfc")
-	register_s3_method("spatstat", "as.ppp", "sf")
-	register_s3_method("spatstat", "as.owin", "POLYGON")
-	register_s3_method("spatstat", "as.owin", "MULTIPOLYGON")
-	register_s3_method("spatstat", "as.owin", "sfc_POLYGON")
-	register_s3_method("spatstat", "as.owin", "sfc_MULTIPOLYGON")
-	register_s3_method("spatstat", "as.owin", "sfc")
-	register_s3_method("spatstat", "as.owin", "sf")
-	register_s3_method("spatstat", "as.psp", "LINESTRING")
-	register_s3_method("spatstat", "as.psp", "MULTILINESTRING")
-	register_s3_method("spatstat", "as.psp", "sfc_MULTILINESTRING")
-	register_s3_method("spatstat", "as.psp", "sfc")
-	register_s3_method("spatstat", "as.psp", "sf")
-	register_s3_method("wk", "as_wkb", "sf")
-	register_s3_method("wk", "as_wkb", "sfc")
-	register_s3_method("wk", "as_wkb", "sfg")
+	register_s3_method("spatstat.geom", "as.ppp", "sfc")
+	register_s3_method("spatstat.geom", "as.ppp", "sf")
+	register_s3_method("spatstat.geom", "as.owin", "POLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "MULTIPOLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "sfc_POLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "sfc_MULTIPOLYGON")
+	register_s3_method("spatstat.geom", "as.owin", "sfc")
+	register_s3_method("spatstat.geom", "as.owin", "sf")
+	register_s3_method("spatstat.geom", "as.psp", "LINESTRING")
+	register_s3_method("spatstat.geom", "as.psp", "MULTILINESTRING")
+	register_s3_method("spatstat.geom", "as.psp", "sfc_MULTILINESTRING")
+	register_s3_method("spatstat.geom", "as.psp", "sfc")
+	register_s3_method("spatstat.geom", "as.psp", "sf")
+	register_s3_method("s2", "as_s2_geography", "sfg")
+	register_s3_method("s2", "as_s2_geography", "sfc")
+	register_s3_method("s2", "as_s2_geography", "sf")
 	register_vctrs_methods()
 }
 
