@@ -54,6 +54,7 @@ Rcpp::List allocate_out_list(OGRFeatureDefn *poFDefn, int n_features, bool int64
 			case OFTRealList:
 			case OFTIntegerList:
 			case OFTInteger64List:
+			case OFTBinary:
 				out[i] = Rcpp::List(n_features);
 				break;
 			case OFTString:
@@ -370,6 +371,17 @@ Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
 					}
 					}
 					break;
+				case OFTBinary: {
+					Rcpp::List lv;
+					lv = out[iField];
+					int n;
+					const GByte *bl = poFeature->GetFieldAsBinary(iField, &n);
+					Rcpp::RawVector rv(n);
+					for (int j = 0; j < rv.size(); j++)
+						rv[j] = bl[j];
+					lv[i] = rv;
+					}
+					break;
 				default: // break through: anything else to be converted to string?
 				case OFTString: {
 					Rcpp::CharacterVector cv;
@@ -457,7 +469,7 @@ Rcpp::List sf_from_ogrlayer(OGRLayer *poLayer, bool quiet, bool int64_as_string,
 		// convert to R:
 		Rcpp::List sfc = sfc_from_ogr(poGeom, false); // don't destroy
 		OGRGeomFieldDefn *fdfn = poFDefn->GetGeomFieldDefn(iGeom);
-		sfc.attr("crs") = get_crs(fdfn->GetSpatialRef()); // overwrite: see #449 for the reason why
+		sfc.attr("crs") = create_crs(fdfn->GetSpatialRef()); // overwrite: see #449 for the reason why
 		out[iGeom + poFDefn->GetFieldCount() + fid_column.size()] = sfc;
 	}
 
@@ -480,9 +492,11 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 		Rcpp::CharacterVector query,
 		Rcpp::CharacterVector options, bool quiet, Rcpp::NumericVector toTypeUser,
 		Rcpp::CharacterVector fid_column_name, Rcpp::CharacterVector drivers,
+		Rcpp::CharacterVector wkt_filter,
 		bool promote_to_multi = true, bool int64_as_string = false,
 		bool dsn_exists = true,
-		bool dsn_isdb = false) {
+		bool dsn_isdb = false,
+		int width = 80) {
 
 	// adapted from the OGR tutorial @ www.gdal.org
 	std::vector <char *> open_options = create_options(options, quiet);
@@ -504,7 +518,7 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 		Rcpp::stop("Cannot open %s; The file doesn't seem to exist.", datasource);
 	}
 
-	if (layer.size() == 0) { // no layer specified
+	if (layer.size() == 0 && Rcpp::CharacterVector::is_na(query[0])) { // no layer specified
 		switch (poDS->GetLayerCount()) {
 			case 0: { // error:
 				Rcpp::stop("No layers in datasource.");
@@ -534,6 +548,8 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 		poLayer = poDS->ExecuteSQL(query[0], NULL, NULL);
 		if (poLayer == NULL)
 			Rcpp::stop("Query execution failed, cannot open layer.\n"); // #nocov
+		if (layer.size())
+			Rcpp::warning("argument layer is ignored when query is specified\n"); // #nocov
 	} else
 		poLayer = 	poDS->GetLayerByName(layer[0]);
 	if (poLayer == NULL) {
@@ -541,9 +557,40 @@ Rcpp::List CPL_read_ogr(Rcpp::CharacterVector datasource, Rcpp::CharacterVector
 		Rcpp::stop("Opening layer failed.\n");
 	}
 
-	if (! quiet)
-		Rcpp::Rcout << "Reading layer `" << layer[0] << "' from data source `" << datasource[0] << // #nocov
-			"' using driver `" << poDS->GetDriverName() << "'" << std::endl;                       // #nocov
+	// set spatial filter?
+	if (wkt_filter.size()) {
+		char *wkt = wkt_filter[0];
+		OGRGeometry *new_geom;
+#if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR <= 2
+		OGRErr err = OGRGeometryFactory::createFromWkt(&wkt, poLayer->GetSpatialRef(), &new_geom);
+#else
+		OGRErr err = OGRGeometryFactory::createFromWkt((const char *) wkt, poLayer->GetSpatialRef(), &new_geom);
+#endif
+		if (err != OGRERR_NONE) {
+			Rcpp::Rcout << "Cannot create geometry from: " << wkt_filter[0] << std::endl;
+			Rcpp::stop("wkt parse error.\n");
+		}
+		poLayer->SetSpatialFilter(new_geom);
+		OGRGeometryFactory::destroyGeometry(new_geom);
+	}
+
+	if (! quiet) {
+		if (! Rcpp::CharacterVector::is_na(query[0]))
+			Rcpp::Rcout << "Reading query `" << query[0] << "' from data source ";
+		else
+			Rcpp::Rcout << "Reading layer `" << layer[0] << "' from data source ";
+		// if (LENGTH(datasource[0]) > (width - (34 + LENGTH(layer[0]))))
+		Rcpp::String ds(datasource(0));
+		if (layer.size()) { 
+			Rcpp::String la(layer(0));
+			if (strlen(ds.get_cstring()) > (width - (34 + strlen(la.get_cstring()))))
+				Rcpp::Rcout << std::endl << "  ";
+		}
+		Rcpp::Rcout << "`" << datasource[0] << "' ";
+		if (((int) strlen(ds.get_cstring())) > (width - 25))
+			Rcpp::Rcout << std::endl << "  ";
+		Rcpp::Rcout << "using driver `" << poDS->GetDriverName() << "'" << std::endl;                       // #nocov
+	}
 
 	Rcpp::List out = sf_from_ogrlayer(poLayer, quiet, int64_as_string, toTypeUser, fid_column_name,
 		promote_to_multi);
