@@ -19,7 +19,9 @@ st_as_sf(d, wkt = "geom", remove = FALSE)
 st_as_sfc(c("POINT(0 0)", "POINT(1 1)"))
 st_as_sfc(c("POINT(0 0)", "POINT(1 1)", "POLYGON((0 0,1 1,0 1,0 0))"))
 st_as_sfc(character(0))
-st_as_sfc(character(0), 4326)
+x = st_as_sfc(character(0), 4326)
+y = st_as_sfc(character(0), crs = 4326)
+all.equal(x, y)
 st_as_sfc(c("POINT(0 0)", "POINT(1 1)", "POLYGON((0 0,1 1,0 1,0 0))"),
 	"+proj=longlat +datum=WGS84")
 dg = st_as_sf(d, wkt = "geom")
@@ -38,9 +40,9 @@ d
 x = st_sfc(list(st_point(0:1), st_point(0:1)), crs = 4326)
 # don't warn when replacing crs with identical value:
 st_sfc(x, crs = 4326)
-st_sfc(x, crs = "+proj=longlat +datum=WGS84 +no_defs")
+y = st_sfc(x, crs = "+proj=longlat +datum=WGS84 +no_defs")
 # but do when it changes:
-st_sfc(x, crs = 3857)
+y = st_sfc(x, crs = 3857)
 
 p = st_point(0:1)
 st_cast(p, "MULTIPOINT")
@@ -96,7 +98,7 @@ try(as(st_sfc(st_linestring(matrix(1:9,3))), "Spatial"))
 # check conus is present:
 x = st_sfc(st_point(c(-90,35)), st_point(c(-80,36)),
 	crs = "+proj=longlat +datum=NAD27")
-st_transform(x, 3857)
+y = st_transform(x, 3857)
 
 sf_extSoftVersion()[1:3]
 
@@ -118,7 +120,8 @@ ll != st_sfc(p_)
 str(x)
 nc = st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE)
 str(nc)
-st_as_sfc(st_bbox(nc))
+bb = st_as_sfc(st_bbox(nc))
+format(st_bbox(nc))
 
 st_agr("constant")
 st_agr()
@@ -168,13 +171,11 @@ b = st_sf(a = 11:14,
  geom = st_sfc(st_point(c(10,10)), st_point(c(2,2)), st_point(c(2,2)), st_point(c(3,3))))
 st_join(a, b)
 st_join(a, b, left = FALSE)
-# deprecated:
-try(x <- st_join(a, b, FUN = mean))
 # st_join, largest = TRUE:
-nc <- st_transform(st_read(system.file("shape/nc.shp", package="sf")), 2264)
+nc <- st_transform(st_read(system.file("shape/nc.shp", package="sf"), quiet = TRUE), 2264)
 gr = st_sf(
     label = apply(expand.grid(1:10, LETTERS[10:1])[,2:1], 1, paste0, collapse = " "),
-    geom = st_make_grid(nc))
+	geom = st_make_grid(st_as_sfc(st_bbox(nc))))
 gr$col = sf.colors(10, categorical = TRUE, alpha = .3)
 # cut, to check, NA's work out:
 gr = gr[-(1:30),]
@@ -191,16 +192,18 @@ suppressWarnings(RNGversion("3.5.3"))
 set.seed(131)
 options(digits=6)
 x = st_sfc(st_polygon(list(rbind(c(0,1),c(90,1),c(90,90),c(0,90),c(0,1)))), crs = st_crs(4326))
-#if (sf_extSoftVersion()["proj.4"] >= "4.9.0")
   (p <- st_sample(x, 10))
+  p <- st_sample(x[[1]], 10) # sfg method
 x = st_sfc(st_polygon(list(rbind(c(0,0),c(90,0),c(90,90),c(0,90),c(0,0))))) # NOT long/lat:
 p <- st_sample(x, 10)
 x = st_sfc(st_polygon(list(rbind(c(-180,-90),c(180,-90),c(180,90),c(-180,90),c(-180,-90)))),
  crs=st_crs(4326))
-if (sf_extSoftVersion()["proj.4"] >= "4.9.0") # lwgeom breaks on this
-  (p <- st_sample(x, 10))
+#FIXME:
+# if (sf_extSoftVersion()["proj.4"] >= "4.9.0") # lwgeom breaks on this
+#   (p <- st_sample(x, 10))
 pt = st_multipoint(matrix(1:20,,2))
 st_sample(p, 3)
+try(st_sample(p, 3.3))
 ls = st_sfc(st_linestring(rbind(c(0,0),c(0,1))),
  st_linestring(rbind(c(0,0),c(.1,0))),
  st_linestring(rbind(c(0,1),c(.1,1))),
@@ -237,10 +240,10 @@ st_jitter(st_sfc(st_point(0:1)), amount = .1)
 # st_bbox:
 library(sp)
 demo(meuse, ask = FALSE, echo = FALSE)
-st_bbox(meuse)
-st_crs(meuse)
+suppressWarnings(st_bbox(meuse))
+crs = suppressWarnings(st_crs(meuse))
 library(raster)
-st_bbox(raster(meuse.grid))
+suppressWarnings(st_bbox(raster(meuse.grid)))
 st_bbox(extent(raster()))
 
 # st_to_s2
@@ -323,5 +326,14 @@ st_union(shape1, shape4)
 st_union(shape4, shape1)
 
 # transform empty:
-st_sf(geom=st_sfc()) %>% st_set_crs(3587) %>% st_transform(4326)
+tr = st_sf(geom=st_sfc()) %>% st_set_crs(3587) %>% st_transform(4326)
 
+# NA values are converted to empty; #1114:
+x <- data.frame(name=LETTERS)
+y <- data.frame(name=LETTERS[1:13], letters[14:26])
+y$geometry <- st_sfc(st_point(c(0,0)))
+y <- st_sf(y)
+out = merge(x, y, all.x=TRUE)
+class(out)
+
+st_as_sf(st_sfc(st_point(0:1)))
